"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeReplacer = exports.transform = exports.waitFinished = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const stream_1 = require("stream");
const pako_1 = __importDefault(require("pako"));
const json_ext_1 = require("@discoveryjs/json-ext");
const _1 = __importStar(require("./"));
function waitFinished(stream) {
    return new Promise((resolve, reject) => {
        stream.once('end', resolve);
        stream.once('finish', resolve);
        stream.once('error', reject);
    });
}
exports.waitFinished = waitFinished;
async function transform(options, from, to) {
    const normalizedFrom = from.map((item) => typeof item === 'string' ? { type: 'filename', filename: item } : item);
    const toDir = path_1.default.dirname(to);
    if (!fs_1.default.existsSync(toDir)) {
        fs_1.default.mkdirSync(toDir, { recursive: true });
    }
    const outputStream = fs_1.default.createWriteStream(to);
    const htmlWriter = new _1.default(options.writer);
    for (const fromItem of normalizedFrom) {
        const id = path_1.default.basename(fromItem.filename);
        let stream;
        if (fromItem.type === 'filename') {
            stream = fs_1.default.createReadStream(fromItem.filename);
        }
        else if (fromItem.type === 'data') {
            stream = (0, json_ext_1.stringifyStream)(fromItem.data, fromItem.replacer);
        }
        else {
            stream = fromItem.stream;
        }
        if (options.writer.dataCompression === false) {
            htmlWriter.addChunkWriter(stream, id);
        }
        else {
            const htmlStats = await (0, json_ext_1.parseChunked)(stream);
            const binaryJSONArray = (0, _1.encodeBinaryJSON)(htmlStats);
            const binaryJSONCompressedArray = pako_1.default.deflate(binaryJSONArray);
            let readIndex = 0;
            const htmlStatsStream = new stream_1.Readable({
                read(size) {
                    const chunk = binaryJSONCompressedArray.subarray(readIndex, readIndex + size);
                    readIndex += size;
                    this.push(Buffer.from(chunk).toString('base64'));
                    if (readIndex >= binaryJSONCompressedArray.length) {
                        this.push(null);
                    }
                },
            });
            htmlWriter.addChunkWriter(htmlStatsStream, id);
        }
    }
    htmlWriter.getStream().pipe(outputStream);
    htmlWriter.write();
    await waitFinished(outputStream);
    return to;
}
exports.transform = transform;
function makeReplacer(from, to = '', ignoreKeys = []) {
    if (!from) {
        return;
    }
    return (key, value) => {
        if (typeof value === 'string' && !ignoreKeys.includes(key)) {
            if (value.includes(from))
                return value.split(from).join(to);
        }
        return value;
    };
}
exports.makeReplacer = makeReplacer;
//# sourceMappingURL=utils.js.map